import { Constants } from "../../metadata/Decorated/types";
import { RpcInterface } from "../../rpc-core/types";
import { Hash, RuntimeVersion } from "../../types/interfaces";
import { InterfaceRegistry } from "../../types/interfaceRegistry";
import { CallFunction, RegistryError, RegistryTypes, SignerPayloadRawBase } from "../../types/types";
import { ApiInterfaceRx, ApiOptions, ApiTypes, DecoratedRpc, DecorateMethod, QueryableStorage, QueryableStorageMulti, SubmittableExtrinsics, Signer } from '../types';
import { Metadata } from "../../types";
import Init from './Init';
interface KeyringSigner {
    sign(message: Uint8Array): Uint8Array;
}
interface SignerRawOptions {
    signer?: Signer;
}
export default abstract class ApiBase<ApiType extends ApiTypes> extends Init<ApiType> {
    /**
     * @description Create an instance of the class
     *
     * @param options Options object to create API instance or a Provider instance
     *
     * @example
     * <BR>
     *
     * ```javascript
     * import Api from '@polkadot/api/promise';
     *
     * const api = new Api().isReady();
     *
     * api.rpc.subscribeNewHeads((header) => {
     *   console.log(`new block #${header.number.toNumber()}`);
     * });
     * ```
     */
    constructor(options: ApiOptions | undefined, type: ApiTypes, decorateMethod: DecorateMethod<ApiType>);
    /**
     * @description Contains the parameter types (constants) of all modules.
     *
     * The values are instances of the appropriate type and are accessible using `section`.`constantName`,
     *
     * @example
     * <BR>
     *
     * ```javascript
     * console.log(api.consts.democracy.enactmentPeriod.toString())
     * ```
     */
    get consts(): Constants;
    /**
     * @description Derived results that are injected into the API, allowing for combinations of various query results.
     *
     * @example
     * <BR>
     *
     * ```javascript
     * api.derive.chain.bestNumber((number) => {
     *   console.log('best number', number);
     * });
     * ```
     */
    get derive(): ReturnType<ApiBase<ApiType>['decorateDerive']>;
    /**
     * @description  Returns the version of extrinsics in-use on this chain
     */
    get extrinsicVersion(): number;
    /**
     * @description Contains the genesis Hash of the attached chain. Apart from being useful to determine the actual chain, it can also be used to sign immortal transactions.
     */
    get genesisHash(): Hash;
    /**
     * @description `true` when subscriptions are supported
     */
    get hasSubscriptions(): boolean;
    /**
     * @description The library information name & version (from package.json)
     */
    get libraryInfo(): string;
    /**
     * @description Contains all the chain state modules and their subsequent methods in the API. These are attached dynamically from the runtime metadata.
     *
     * All calls inside the namespace, is denoted by `section`.`method` and may take an optional query parameter. As an example, `api.query.timestamp.now()` (current block timestamp) does not take parameters, while `api.query.system.accountNonce(<accountId>)` (retrieving the associated nonce for an account), takes the `AccountId` as a parameter.
     *
     * @example
     * <BR>
     *
     * ```javascript
     * api.query.balances.freeBalance(<accountId>, (balance) => {
     *   console.log('new balance', balance);
     * });
     * ```
     */
    get query(): QueryableStorage<ApiType>;
    /**
     * @description Allows for the querying of multiple storage entries and the combination thereof into a single result. This is a very optimal way to make multiple queries since it only makes a single connection to the node and retrieves the data over one subscription.
     *
     * @example
     * <BR>
     *
     * ```javascript
     * const unsub = await api.queryMulti(
     *   [
     *     // you can include the storage without any parameters
     *     api.query.balances.totalIssuance,
     *     // or you can pass parameters to the storage query
     *     [api.query.balances.freeBalance, '5GrwvaEF5zXb26Fz9rcQpDWS57CtERHpNehXCPcNoHGKutQY']
     *   ],
     *   ([existential, balance]) => {
     *     console.log(`You have ${balance.sub(existential)} more than the existential deposit`);
     *
     *     unsub();
     *   }
     * );
     * ```
     */
    get queryMulti(): QueryableStorageMulti<ApiType>;
    /**
     * @description Contains all the raw rpc sections and their subsequent methods in the API as defined by the jsonrpc interface definitions. Unlike the dynamic `api.query` and `api.tx` sections, these methods are fixed (although extensible with node upgrades) and not determined by the runtime.
     *
     * RPC endpoints available here allow for the query of chain, node and system information, in addition to providing interfaces for the raw queries of state (using known keys) and the submission of transactions.
     *
     * @example
     * <BR>
     *
     * ```javascript
     * api.rpc.chain.subscribeNewHeads((header) => {
     *   console.log('new header', header);
     * });
     * ```
     */
    get rpc(): DecoratedRpc<ApiType, RpcInterface>;
    /**
     * @description Yields the current attached runtime metadata. Generally this is only used to construct extrinsics & storage, but is useful for current runtime inspection.
     */
    get runtimeMetadata(): Metadata;
    /**
     * @description Contains the version information for the current runtime.
     */
    get runtimeVersion(): RuntimeVersion;
    /**
     * @description The underlying Rx API interface
     */
    get rx(): Pick<ApiInterfaceRx, 'tx' | 'rpc'>;
    /**
     * @description The type of this API instance, either 'rxjs' or 'promise'
     */
    get type(): ApiTypes;
    /**
     * @description Contains all the extrinsic modules and their subsequent methods in the API. It allows for the construction of transactions and the submission thereof. These are attached dynamically from the runtime metadata.
     *
     * @example
     * <BR>
     *
     * ```javascript
     * api.tx.balances
     *   .transfer(<recipientId>, <balance>)
     *   .signAndSend(<keyPair>, ({status}) => {
     *     console.log('tx status', status.asFinalized.toHex());
     *   });
     * ```
     */
    get tx(): SubmittableExtrinsics<ApiType>;
    /**
     * @description Creates an instance of a type as registered
     */
    createType: <K extends "bool" | "Option<bool>" | "Vec<bool>" | "Bytes" | "Option<Bytes>" | "Vec<Bytes>" | "Data" | "Option<Data>" | "Vec<Data>" | "H160" | "Option<H160>" | "Vec<H160>" | "H256" | "Option<H256>" | "Vec<H256>" | "H512" | "Option<H512>" | "Vec<H512>" | "IdentityFields" | "Option<IdentityFields>" | "Vec<IdentityFields>" | "i8" | "Option<i8>" | "Vec<i8>" | "i16" | "Option<i16>" | "Vec<i16>" | "i32" | "Option<i32>" | "Vec<i32>" | "i64" | "Option<i64>" | "Vec<i64>" | "Fixed64" | "Option<Fixed64>" | "Vec<Fixed64>" | "i128" | "Option<i128>" | "Vec<i128>" | "i256" | "Option<i256>" | "Vec<i256>" | "Null" | "Option<Null>" | "Vec<Null>" | "StorageData" | "Option<StorageData>" | "Vec<StorageData>" | "StorageKey" | "Option<StorageKey>" | "Vec<StorageKey>" | "Text" | "Option<Text>" | "Vec<Text>" | "Type" | "Option<Type>" | "Vec<Type>" | "u8" | "Compact<u8>" | "Option<u8>" | "Vec<u8>" | "u16" | "Compact<u16>" | "Option<u16>" | "Vec<u16>" | "u32" | "Compact<u32>" | "Option<u32>" | "Vec<u32>" | "u64" | "Compact<u64>" | "Option<u64>" | "Vec<u64>" | "u128" | "Compact<u128>" | "Option<u128>" | "Vec<u128>" | "u256" | "Compact<u256>" | "Option<u256>" | "Vec<u256>" | "U256" | "Compact<U256>" | "Option<U256>" | "Vec<U256>" | "usize" | "Compact<usize>" | "Option<usize>" | "Vec<usize>" | "Raw" | "Option<Raw>" | "Vec<Raw>" | "Extrinsic" | "Option<Extrinsic>" | "Vec<Extrinsic>" | "ExtrinsicEra" | "Option<ExtrinsicEra>" | "Vec<ExtrinsicEra>" | "ExtrinsicPayload" | "Option<ExtrinsicPayload>" | "Vec<ExtrinsicPayload>" | "ExtrinsicV1" | "Option<ExtrinsicV1>" | "Vec<ExtrinsicV1>" | "ExtrinsicPayloadV1" | "Option<ExtrinsicPayloadV1>" | "Vec<ExtrinsicPayloadV1>" | "ExtrinsicSignatureV1" | "Option<ExtrinsicSignatureV1>" | "Vec<ExtrinsicSignatureV1>" | "ExtrinsicV2" | "Option<ExtrinsicV2>" | "Vec<ExtrinsicV2>" | "ExtrinsicPayloadV2" | "Option<ExtrinsicPayloadV2>" | "Vec<ExtrinsicPayloadV2>" | "ExtrinsicSignatureV2" | "Option<ExtrinsicSignatureV2>" | "Vec<ExtrinsicSignatureV2>" | "ExtrinsicV3" | "Option<ExtrinsicV3>" | "Vec<ExtrinsicV3>" | "ExtrinsicPayloadV3" | "Option<ExtrinsicPayloadV3>" | "Vec<ExtrinsicPayloadV3>" | "ExtrinsicSignatureV3" | "Option<ExtrinsicSignatureV3>" | "Vec<ExtrinsicSignatureV3>" | "ExtrinsicV4" | "Option<ExtrinsicV4>" | "Vec<ExtrinsicV4>" | "ExtrinsicPayloadV4" | "Option<ExtrinsicPayloadV4>" | "Vec<ExtrinsicPayloadV4>" | "ExtrinsicSignatureV4" | "Option<ExtrinsicSignatureV4>" | "Vec<ExtrinsicSignatureV4>" | "ExtrinsicUnknown" | "Option<ExtrinsicUnknown>" | "Vec<ExtrinsicUnknown>" | "ExtrinsicPayloadUnknown" | "Option<ExtrinsicPayloadUnknown>" | "Vec<ExtrinsicPayloadUnknown>" | "AccountId" | "Option<AccountId>" | "Vec<AccountId>" | "AccountIdOf" | "Option<AccountIdOf>" | "Vec<AccountIdOf>" | "AccountIndex" | "Compact<AccountIndex>" | "Option<AccountIndex>" | "Vec<AccountIndex>" | "Address" | "Option<Address>" | "Vec<Address>" | "AssetId" | "Compact<AssetId>" | "Option<AssetId>" | "Vec<AssetId>" | "Balance" | "Compact<Balance>" | "Option<Balance>" | "Vec<Balance>" | "BalanceOf" | "Option<BalanceOf>" | "Vec<BalanceOf>" | "Block" | "Option<Block>" | "Vec<Block>" | "BlockNumber" | "Compact<BlockNumber>" | "Option<BlockNumber>" | "Vec<BlockNumber>" | "Call" | "Option<Call>" | "Vec<Call>" | "ChangesTrieConfiguration" | "Option<ChangesTrieConfiguration>" | "Vec<ChangesTrieConfiguration>" | "ConsensusEngineId" | "Compact<ConsensusEngineId>" | "Option<ConsensusEngineId>" | "Vec<ConsensusEngineId>" | "Digest" | "Option<Digest>" | "Vec<Digest>" | "DigestItem" | "Option<DigestItem>" | "Vec<DigestItem>" | "DispatchClass" | "Option<DispatchClass>" | "Vec<DispatchClass>" | "DispatchInfo" | "Option<DispatchInfo>" | "Vec<DispatchInfo>" | "DispatchInfoTo190" | "Option<DispatchInfoTo190>" | "Vec<DispatchInfoTo190>" | "Hash" | "Option<Hash>" | "Vec<Hash>" | "Header" | "Option<Header>" | "Vec<Header>" | "Index" | "Compact<Index>" | "Option<Index>" | "Vec<Index>" | "Justification" | "Option<Justification>" | "Vec<Justification>" | "KeyValue" | "Option<KeyValue>" | "Vec<KeyValue>" | "KeyTypeId" | "Compact<KeyTypeId>" | "Option<KeyTypeId>" | "Vec<KeyTypeId>" | "LockIdentifier" | "Option<LockIdentifier>" | "Vec<LockIdentifier>" | "Moment" | "Compact<Moment>" | "Option<Moment>" | "Vec<Moment>" | "Origin" | "Option<Origin>" | "Vec<Origin>" | "Perbill" | "Compact<Perbill>" | "Option<Perbill>" | "Vec<Perbill>" | "Percent" | "Compact<Percent>" | "Option<Percent>" | "Vec<Percent>" | "Permill" | "Compact<Permill>" | "Option<Permill>" | "Vec<Permill>" | "Perquintill" | "Compact<Perquintill>" | "Option<Perquintill>" | "Vec<Perquintill>" | "Phantom" | "Option<Phantom>" | "Vec<Phantom>" | "PhantomData" | "Option<PhantomData>" | "Vec<PhantomData>" | "Signature" | "Option<Signature>" | "Vec<Signature>" | "SignedBlock" | "Option<SignedBlock>" | "Vec<SignedBlock>" | "SignerPayload" | "Option<SignerPayload>" | "Vec<SignerPayload>" | "ValidatorId" | "Option<ValidatorId>" | "Vec<ValidatorId>" | "Weight" | "Compact<Weight>" | "Option<Weight>" | "Vec<Weight>" | "WeightMultiplier" | "Option<WeightMultiplier>" | "Vec<WeightMultiplier>" | "Ed25519Signature" | "Option<Ed25519Signature>" | "Vec<Ed25519Signature>" | "Sr25519Signature" | "Option<Sr25519Signature>" | "Vec<Sr25519Signature>" | "EcdsaSignature" | "Option<EcdsaSignature>" | "Vec<EcdsaSignature>" | "MultiSignature" | "Option<MultiSignature>" | "Vec<MultiSignature>" | "ImmortalEra" | "Option<ImmortalEra>" | "Vec<ImmortalEra>" | "MortalEra" | "Option<MortalEra>" | "Vec<MortalEra>" | "PreRuntime" | "Option<PreRuntime>" | "Vec<PreRuntime>" | "SealV0" | "Option<SealV0>" | "Vec<SealV0>" | "Seal" | "Option<Seal>" | "Vec<Seal>" | "Consensus" | "Option<Consensus>" | "Vec<Consensus>" | "InclusionHeight" | "Option<InclusionHeight>" | "Vec<InclusionHeight>" | "Uncle" | "Option<Uncle>" | "Vec<Uncle>" | "UncleEntryItem" | "Option<UncleEntryItem>" | "Vec<UncleEntryItem>" | "RawAuraPreDigest" | "Option<RawAuraPreDigest>" | "Vec<RawAuraPreDigest>" | "BabeAuthorityWeight" | "Compact<BabeAuthorityWeight>" | "Option<BabeAuthorityWeight>" | "Vec<BabeAuthorityWeight>" | "BabeBlockWeight" | "Compact<BabeBlockWeight>" | "Option<BabeBlockWeight>" | "Vec<BabeBlockWeight>" | "MaybeVrf" | "Option<MaybeVrf>" | "Vec<MaybeVrf>" | "BabeWeight" | "Compact<BabeWeight>" | "Option<BabeWeight>" | "Vec<BabeWeight>" | "RawBabePreDigest" | "Option<RawBabePreDigest>" | "Vec<RawBabePreDigest>" | "RawBabePreDigestPrimary" | "Option<RawBabePreDigestPrimary>" | "Vec<RawBabePreDigestPrimary>" | "RawBabePreDigestSecondary" | "Option<RawBabePreDigestSecondary>" | "Vec<RawBabePreDigestSecondary>" | "RawBabePreDigestTo159" | "Option<RawBabePreDigestTo159>" | "Vec<RawBabePreDigestTo159>" | "RawBabePreDigestPrimaryTo159" | "Option<RawBabePreDigestPrimaryTo159>" | "Vec<RawBabePreDigestPrimaryTo159>" | "RawBabePreDigestSecondaryTo159" | "Option<RawBabePreDigestSecondaryTo159>" | "Vec<RawBabePreDigestSecondaryTo159>" | "RawBabePreDigestCompat" | "Option<RawBabePreDigestCompat>" | "Vec<RawBabePreDigestCompat>" | "SlotNumber" | "Compact<SlotNumber>" | "Option<SlotNumber>" | "Vec<SlotNumber>" | "VrfData" | "Option<VrfData>" | "Vec<VrfData>" | "VrfProof" | "Option<VrfProof>" | "Vec<VrfProof>" | "AccountData" | "Option<AccountData>" | "Vec<AccountData>" | "BalanceLockTo212" | "Option<BalanceLockTo212>" | "Vec<BalanceLockTo212>" | "BalanceLock" | "Option<BalanceLock>" | "Vec<BalanceLock>" | "Reasons" | "Option<Reasons>" | "Vec<Reasons>" | "VestingSchedule" | "Option<VestingSchedule>" | "Vec<VestingSchedule>" | "WithdrawReasons" | "Option<WithdrawReasons>" | "Vec<WithdrawReasons>" | "MemberCount" | "Compact<MemberCount>" | "Option<MemberCount>" | "Vec<MemberCount>" | "ProposalIndex" | "Compact<ProposalIndex>" | "Option<ProposalIndex>" | "Vec<ProposalIndex>" | "Votes" | "Option<Votes>" | "Vec<Votes>" | "AuthorityId" | "Option<AuthorityId>" | "Vec<AuthorityId>" | "AliveContractInfo" | "Option<AliveContractInfo>" | "Vec<AliveContractInfo>" | "CodeHash" | "Option<CodeHash>" | "Vec<CodeHash>" | "ContractCallRequest" | "Option<ContractCallRequest>" | "Vec<ContractCallRequest>" | "ContractExecResultSuccess" | "Option<ContractExecResultSuccess>" | "Vec<ContractExecResultSuccess>" | "ContractExecResult" | "Option<ContractExecResult>" | "Vec<ContractExecResult>" | "ContractInfo" | "Option<ContractInfo>" | "Vec<ContractInfo>" | "ContractStorageKey" | "Option<ContractStorageKey>" | "Vec<ContractStorageKey>" | "Gas" | "Compact<Gas>" | "Option<Gas>" | "Vec<Gas>" | "PrefabWasmModule" | "Option<PrefabWasmModule>" | "Vec<PrefabWasmModule>" | "PrefabWasmModuleReserved" | "Option<PrefabWasmModuleReserved>" | "Vec<PrefabWasmModuleReserved>" | "ScheduleTo212" | "Option<ScheduleTo212>" | "Vec<ScheduleTo212>" | "Schedule" | "Option<Schedule>" | "Vec<Schedule>" | "SeedOf" | "Option<SeedOf>" | "Vec<SeedOf>" | "TombstoneContractInfo" | "Option<TombstoneContractInfo>" | "Vec<TombstoneContractInfo>" | "TrieId" | "Option<TrieId>" | "Vec<TrieId>" | "Conviction" | "Option<Conviction>" | "Vec<Conviction>" | "PropIndex" | "Compact<PropIndex>" | "Option<PropIndex>" | "Vec<PropIndex>" | "Proposal" | "Option<Proposal>" | "Vec<Proposal>" | "ReferendumIndex" | "Compact<ReferendumIndex>" | "Option<ReferendumIndex>" | "Vec<ReferendumIndex>" | "ReferendumInfo" | "Option<ReferendumInfo>" | "Vec<ReferendumInfo>" | "NewAccountOutcome" | "Option<NewAccountOutcome>" | "Vec<NewAccountOutcome>" | "Amount" | "Option<Amount>" | "Vec<Amount>" | "AssetOf" | "Compact<AssetOf>" | "Option<AssetOf>" | "Vec<AssetOf>" | "AccountInfo" | "Option<AccountInfo>" | "Vec<AccountInfo>" | "LockPeriods" | "Option<LockPeriods>" | "Vec<LockPeriods>" | "InherentOfflineReport" | "Option<InherentOfflineReport>" | "Vec<InherentOfflineReport>" | "SessionKey" | "Option<SessionKey>" | "Vec<SessionKey>" | "OpaqueKey" | "Option<OpaqueKey>" | "Vec<OpaqueKey>" | "ApprovalFlag" | "Compact<ApprovalFlag>" | "Option<ApprovalFlag>" | "Vec<ApprovalFlag>" | "SetIndex" | "Compact<SetIndex>" | "Option<SetIndex>" | "Vec<SetIndex>" | "Vote" | "Option<Vote>" | "Vec<Vote>" | "VoteIndex" | "Compact<VoteIndex>" | "Option<VoteIndex>" | "Vec<VoteIndex>" | "VoterInfo" | "Option<VoterInfo>" | "Vec<VoterInfo>" | "VoteThreshold" | "Option<VoteThreshold>" | "Vec<VoteThreshold>" | "Account" | "Option<Account>" | "Vec<Account>" | "Log" | "Option<Log>" | "Vec<Log>" | "AssetOptions" | "Option<AssetOptions>" | "Vec<AssetOptions>" | "Owner" | "Option<Owner>" | "Vec<Owner>" | "PermissionsV1" | "Option<PermissionsV1>" | "Vec<PermissionsV1>" | "PermissionVersions" | "Option<PermissionVersions>" | "Vec<PermissionVersions>" | "PermissionLatest" | "Option<PermissionLatest>" | "Vec<PermissionLatest>" | "AuthorityIndex" | "Compact<AuthorityIndex>" | "Option<AuthorityIndex>" | "Vec<AuthorityIndex>" | "AuthorityList" | "Option<AuthorityList>" | "Vec<AuthorityList>" | "AuthorityWeight" | "Compact<AuthorityWeight>" | "Option<AuthorityWeight>" | "Vec<AuthorityWeight>" | "NextAuthority" | "Option<NextAuthority>" | "Vec<NextAuthority>" | "PendingPause" | "Option<PendingPause>" | "Vec<PendingPause>" | "PendingResume" | "Option<PendingResume>" | "Vec<PendingResume>" | "SetId" | "Compact<SetId>" | "Option<SetId>" | "Vec<SetId>" | "StoredPendingChange" | "Option<StoredPendingChange>" | "Vec<StoredPendingChange>" | "StoredState" | "Option<StoredState>" | "Vec<StoredState>" | "IdentityInfoAdditional" | "Option<IdentityInfoAdditional>" | "Vec<IdentityInfoAdditional>" | "IdentityInfo" | "Option<IdentityInfo>" | "Vec<IdentityInfo>" | "IdentityJudgement" | "Option<IdentityJudgement>" | "Vec<IdentityJudgement>" | "RegistrationJudgement" | "Option<RegistrationJudgement>" | "Vec<RegistrationJudgement>" | "Registration" | "Option<Registration>" | "Vec<Registration>" | "RegistrarIndex" | "Compact<RegistrarIndex>" | "Option<RegistrarIndex>" | "Vec<RegistrarIndex>" | "RegistrarInfo" | "Option<RegistrarInfo>" | "Vec<RegistrarInfo>" | "AuthIndex" | "Compact<AuthIndex>" | "Option<AuthIndex>" | "Vec<AuthIndex>" | "AuthoritySignature" | "Option<AuthoritySignature>" | "Vec<AuthoritySignature>" | "Heartbeat" | "Option<Heartbeat>" | "Vec<Heartbeat>" | "OpaqueMultiaddr" | "Option<OpaqueMultiaddr>" | "Vec<OpaqueMultiaddr>" | "OpaquePeerId" | "Option<OpaquePeerId>" | "Vec<OpaquePeerId>" | "OpaqueNetworkState" | "Option<OpaqueNetworkState>" | "Vec<OpaqueNetworkState>" | "Kind" | "Option<Kind>" | "Vec<Kind>" | "OffenceDetails" | "Option<OffenceDetails>" | "Vec<OffenceDetails>" | "Offender" | "Option<Offender>" | "Vec<Offender>" | "OpaqueTimeSlot" | "Option<OpaqueTimeSlot>" | "Vec<OpaqueTimeSlot>" | "ReportIdOf" | "Option<ReportIdOf>" | "Vec<ReportIdOf>" | "Reporter" | "Option<Reporter>" | "Vec<Reporter>" | "ActiveRecovery" | "Option<ActiveRecovery>" | "Vec<ActiveRecovery>" | "RecoveryConfig" | "Option<RecoveryConfig>" | "Vec<RecoveryConfig>" | "Keys" | "Option<Keys>" | "Vec<Keys>" | "SessionKeys1" | "Option<SessionKeys1>" | "Vec<SessionKeys1>" | "SessionKeys2" | "Option<SessionKeys2>" | "Vec<SessionKeys2>" | "SessionKeys3" | "Option<SessionKeys3>" | "Vec<SessionKeys3>" | "SessionKeys4" | "Option<SessionKeys4>" | "Vec<SessionKeys4>" | "SessionKeys5" | "Option<SessionKeys5>" | "Vec<SessionKeys5>" | "SessionKeys6" | "Option<SessionKeys6>" | "Vec<SessionKeys6>" | "FullIdentification" | "Option<FullIdentification>" | "Vec<FullIdentification>" | "IdentificationTuple" | "Option<IdentificationTuple>" | "Vec<IdentificationTuple>" | "SessionIndex" | "Compact<SessionIndex>" | "Option<SessionIndex>" | "Vec<SessionIndex>" | "Bid" | "Option<Bid>" | "Vec<Bid>" | "BidKindVouch" | "Option<BidKindVouch>" | "Vec<BidKindVouch>" | "BidKind" | "Option<BidKind>" | "Vec<BidKind>" | "SocietyJudgement" | "Option<SocietyJudgement>" | "Vec<SocietyJudgement>" | "SocietyVote" | "Option<SocietyVote>" | "Vec<SocietyVote>" | "StrikeCount" | "Compact<StrikeCount>" | "Option<StrikeCount>" | "Vec<StrikeCount>" | "VouchingStatus" | "Option<VouchingStatus>" | "Vec<VouchingStatus>" | "EraIndex" | "Compact<EraIndex>" | "Option<EraIndex>" | "Vec<EraIndex>" | "EraPoints" | "Option<EraPoints>" | "Vec<EraPoints>" | "EraRewards" | "Option<EraRewards>" | "Vec<EraRewards>" | "Exposure" | "Option<Exposure>" | "Vec<Exposure>" | "Forcing" | "Option<Forcing>" | "Vec<Forcing>" | "IndividualExposure" | "Option<IndividualExposure>" | "Vec<IndividualExposure>" | "MomentOf" | "Option<MomentOf>" | "Vec<MomentOf>" | "Nominations" | "Option<Nominations>" | "Vec<Nominations>" | "Points" | "Compact<Points>" | "Option<Points>" | "Vec<Points>" | "RewardDestination" | "Option<RewardDestination>" | "Vec<RewardDestination>" | "SlashJournalEntry" | "Option<SlashJournalEntry>" | "Vec<SlashJournalEntry>" | "SlashingSpansTo204" | "Option<SlashingSpansTo204>" | "Vec<SlashingSpansTo204>" | "SlashingSpans" | "Option<SlashingSpans>" | "Vec<SlashingSpans>" | "SpanIndex" | "Compact<SpanIndex>" | "Option<SpanIndex>" | "Vec<SpanIndex>" | "SpanRecord" | "Option<SpanRecord>" | "Vec<SpanRecord>" | "StakingLedger" | "Option<StakingLedger>" | "Vec<StakingLedger>" | "UnappliedSlashOther" | "Option<UnappliedSlashOther>" | "Vec<UnappliedSlashOther>" | "UnappliedSlash" | "Option<UnappliedSlash>" | "Vec<UnappliedSlash>" | "UnlockChunk" | "Option<UnlockChunk>" | "Vec<UnlockChunk>" | "ValidatorPrefs" | "Option<ValidatorPrefs>" | "Vec<ValidatorPrefs>" | "ValidatorPrefsTo196" | "Option<ValidatorPrefsTo196>" | "Vec<ValidatorPrefsTo196>" | "ValidatorPrefsTo145" | "Option<ValidatorPrefsTo145>" | "Vec<ValidatorPrefsTo145>" | "DigestOf" | "Option<DigestOf>" | "Vec<DigestOf>" | "DispatchError" | "Option<DispatchError>" | "Vec<DispatchError>" | "DispatchErrorModule" | "Option<DispatchErrorModule>" | "Vec<DispatchErrorModule>" | "DispatchErrorTo198" | "Option<DispatchErrorTo198>" | "Vec<DispatchErrorTo198>" | "DispatchResult" | "Option<DispatchResult>" | "Vec<DispatchResult>" | "DispatchResultOf" | "Option<DispatchResultOf>" | "Vec<DispatchResultOf>" | "DispatchResultTo198" | "Option<DispatchResultTo198>" | "Vec<DispatchResultTo198>" | "Event" | "Option<Event>" | "Vec<Event>" | "EventId" | "Option<EventId>" | "Vec<EventId>" | "EventIndex" | "Compact<EventIndex>" | "Option<EventIndex>" | "Vec<EventIndex>" | "EventRecord" | "Option<EventRecord>" | "Vec<EventRecord>" | "EventRecordTo76" | "Option<EventRecordTo76>" | "Vec<EventRecordTo76>" | "Key" | "Option<Key>" | "Vec<Key>" | "Phase" | "Option<Phase>" | "Vec<Phase>" | "OpenTipFinder" | "Option<OpenTipFinder>" | "Vec<OpenTipFinder>" | "OpenTipTip" | "Option<OpenTipTip>" | "Vec<OpenTipTip>" | "OpenTip" | "Option<OpenTip>" | "Vec<OpenTip>" | "TreasuryProposal" | "Option<TreasuryProposal>" | "Vec<TreasuryProposal>" | "Multiplier" | "Option<Multiplier>" | "Vec<Multiplier>" | "Timepoint" | "Option<Timepoint>" | "Vec<Timepoint>" | "Multisig" | "Option<Multisig>" | "Vec<Multisig>" | "VestingInfo" | "Option<VestingInfo>" | "Vec<VestingInfo>" | "BlockAttestations" | "Option<BlockAttestations>" | "Vec<BlockAttestations>" | "IncludedBlocks" | "Option<IncludedBlocks>" | "Vec<IncludedBlocks>" | "MoreAttestations" | "Option<MoreAttestations>" | "Vec<MoreAttestations>" | "EthereumAddress" | "Option<EthereumAddress>" | "Vec<EthereumAddress>" | "AttestedCandidate" | "Option<AttestedCandidate>" | "Vec<AttestedCandidate>" | "AuctionIndex" | "Compact<AuctionIndex>" | "Option<AuctionIndex>" | "Vec<AuctionIndex>" | "BalanceUpload" | "Option<BalanceUpload>" | "Vec<BalanceUpload>" | "Bidder" | "Option<Bidder>" | "Vec<Bidder>" | "CandidateReceipt" | "Option<CandidateReceipt>" | "Vec<CandidateReceipt>" | "CollatorId" | "Option<CollatorId>" | "Vec<CollatorId>" | "CollatorSignature" | "Option<CollatorSignature>" | "Vec<CollatorSignature>" | "EgressQueueRoot" | "Option<EgressQueueRoot>" | "Vec<EgressQueueRoot>" | "HeadData" | "Option<HeadData>" | "Vec<HeadData>" | "IncomingParachainDeploy" | "Option<IncomingParachainDeploy>" | "Vec<IncomingParachainDeploy>" | "IncomingParachainFixed" | "Option<IncomingParachainFixed>" | "Vec<IncomingParachainFixed>" | "IncomingParachain" | "Option<IncomingParachain>" | "Vec<IncomingParachain>" | "LeasePeriod" | "Option<LeasePeriod>" | "Vec<LeasePeriod>" | "LeasePeriodOf" | "Option<LeasePeriodOf>" | "Vec<LeasePeriodOf>" | "NewBidder" | "Option<NewBidder>" | "Vec<NewBidder>" | "ParaId" | "Compact<ParaId>" | "Option<ParaId>" | "Vec<ParaId>" | "ParaIdOf" | "Option<ParaIdOf>" | "Vec<ParaIdOf>" | "ParaInfo" | "Option<ParaInfo>" | "Vec<ParaInfo>" | "ParachainDispatchOrigin" | "Option<ParachainDispatchOrigin>" | "Vec<ParachainDispatchOrigin>" | "ParaScheduling" | "Option<ParaScheduling>" | "Vec<ParaScheduling>" | "Retriable" | "Option<Retriable>" | "Vec<Retriable>" | "SlotRange" | "Option<SlotRange>" | "Vec<SlotRange>" | "SubId" | "Compact<SubId>" | "Option<SubId>" | "Vec<SubId>" | "UpwardMessage" | "Option<UpwardMessage>" | "Vec<UpwardMessage>" | "ValidityAttestation" | "Option<ValidityAttestation>" | "Vec<ValidityAttestation>" | "ValidatorIndex" | "Compact<ValidatorIndex>" | "Option<ValidatorIndex>" | "Vec<ValidatorIndex>" | "ValidityVote" | "Option<ValidityVote>" | "Vec<ValidityVote>" | "WinningDataEntry" | "Option<WinningDataEntry>" | "Vec<WinningDataEntry>" | "WinningData" | "Option<WinningData>" | "Vec<WinningData>" | "CallMetadataV0" | "Option<CallMetadataV0>" | "Vec<CallMetadataV0>" | "EventMetadataV0" | "Option<EventMetadataV0>" | "Vec<EventMetadataV0>" | "FunctionArgumentMetadataV0" | "Option<FunctionArgumentMetadataV0>" | "Vec<FunctionArgumentMetadataV0>" | "FunctionMetadataV0" | "Option<FunctionMetadataV0>" | "Vec<FunctionMetadataV0>" | "MapTypeV0" | "Option<MapTypeV0>" | "Vec<MapTypeV0>" | "MetadataV0" | "Option<MetadataV0>" | "Vec<MetadataV0>" | "ModuleMetadataV0" | "Option<ModuleMetadataV0>" | "Vec<ModuleMetadataV0>" | "OuterDispatchCallV0" | "Option<OuterDispatchCallV0>" | "Vec<OuterDispatchCallV0>" | "OuterDispatchMetadataV0" | "Option<OuterDispatchMetadataV0>" | "Vec<OuterDispatchMetadataV0>" | "OuterEventEventMetadataEventsV0" | "Option<OuterEventEventMetadataEventsV0>" | "Vec<OuterEventEventMetadataEventsV0>" | "OuterEventEventMetadataV0" | "Option<OuterEventEventMetadataV0>" | "Vec<OuterEventEventMetadataV0>" | "OuterEventMetadataV0" | "Option<OuterEventMetadataV0>" | "Vec<OuterEventMetadataV0>" | "PlainTypeV0" | "Option<PlainTypeV0>" | "Vec<PlainTypeV0>" | "RuntimeModuleMetadataV0" | "Option<RuntimeModuleMetadataV0>" | "Vec<RuntimeModuleMetadataV0>" | "StorageFunctionMetadataV0" | "Option<StorageFunctionMetadataV0>" | "Vec<StorageFunctionMetadataV0>" | "StorageFunctionModifierV0" | "Option<StorageFunctionModifierV0>" | "Vec<StorageFunctionModifierV0>" | "StorageFunctionTypeV0" | "Option<StorageFunctionTypeV0>" | "Vec<StorageFunctionTypeV0>" | "StorageMetadataV0" | "Option<StorageMetadataV0>" | "Vec<StorageMetadataV0>" | "EventMetadataV1" | "Option<EventMetadataV1>" | "Vec<EventMetadataV1>" | "FunctionArgumentMetadataV1" | "Option<FunctionArgumentMetadataV1>" | "Vec<FunctionArgumentMetadataV1>" | "FunctionMetadataV1" | "Option<FunctionMetadataV1>" | "Vec<FunctionMetadataV1>" | "MetadataV1" | "Option<MetadataV1>" | "Vec<MetadataV1>" | "ModuleMetadataV1" | "Option<ModuleMetadataV1>" | "Vec<ModuleMetadataV1>" | "StorageFunctionMetadataV1" | "Option<StorageFunctionMetadataV1>" | "Vec<StorageFunctionMetadataV1>" | "StorageFunctionModifierV1" | "Option<StorageFunctionModifierV1>" | "Vec<StorageFunctionModifierV1>" | "StorageFunctionTypeV1" | "Option<StorageFunctionTypeV1>" | "Vec<StorageFunctionTypeV1>" | "EventMetadataV2" | "Option<EventMetadataV2>" | "Vec<EventMetadataV2>" | "FunctionArgumentMetadataV2" | "Option<FunctionArgumentMetadataV2>" | "Vec<FunctionArgumentMetadataV2>" | "FunctionMetadataV2" | "Option<FunctionMetadataV2>" | "Vec<FunctionMetadataV2>" | "MapTypeV2" | "Option<MapTypeV2>" | "Vec<MapTypeV2>" | "MetadataV2" | "Option<MetadataV2>" | "Vec<MetadataV2>" | "ModuleMetadataV2" | "Option<ModuleMetadataV2>" | "Vec<ModuleMetadataV2>" | "PlainTypeV2" | "Option<PlainTypeV2>" | "Vec<PlainTypeV2>" | "StorageFunctionMetadataV2" | "Option<StorageFunctionMetadataV2>" | "Vec<StorageFunctionMetadataV2>" | "StorageFunctionModifierV2" | "Option<StorageFunctionModifierV2>" | "Vec<StorageFunctionModifierV2>" | "StorageFunctionTypeV2" | "Option<StorageFunctionTypeV2>" | "Vec<StorageFunctionTypeV2>" | "DoubleMapTypeV3" | "Option<DoubleMapTypeV3>" | "Vec<DoubleMapTypeV3>" | "EventMetadataV3" | "Option<EventMetadataV3>" | "Vec<EventMetadataV3>" | "FunctionArgumentMetadataV3" | "Option<FunctionArgumentMetadataV3>" | "Vec<FunctionArgumentMetadataV3>" | "FunctionMetadataV3" | "Option<FunctionMetadataV3>" | "Vec<FunctionMetadataV3>" | "MapTypeV3" | "Option<MapTypeV3>" | "Vec<MapTypeV3>" | "MetadataV3" | "Option<MetadataV3>" | "Vec<MetadataV3>" | "ModuleMetadataV3" | "Option<ModuleMetadataV3>" | "Vec<ModuleMetadataV3>" | "PlainTypeV3" | "Option<PlainTypeV3>" | "Vec<PlainTypeV3>" | "StorageFunctionMetadataV3" | "Option<StorageFunctionMetadataV3>" | "Vec<StorageFunctionMetadataV3>" | "StorageFunctionModifierV3" | "Option<StorageFunctionModifierV3>" | "Vec<StorageFunctionModifierV3>" | "StorageFunctionTypeV3" | "Option<StorageFunctionTypeV3>" | "Vec<StorageFunctionTypeV3>" | "DoubleMapTypeV4" | "Option<DoubleMapTypeV4>" | "Vec<DoubleMapTypeV4>" | "EventMetadataV4" | "Option<EventMetadataV4>" | "Vec<EventMetadataV4>" | "FunctionArgumentMetadataV4" | "Option<FunctionArgumentMetadataV4>" | "Vec<FunctionArgumentMetadataV4>" | "FunctionMetadataV4" | "Option<FunctionMetadataV4>" | "Vec<FunctionMetadataV4>" | "MapTypeV4" | "Option<MapTypeV4>" | "Vec<MapTypeV4>" | "MetadataV4" | "Option<MetadataV4>" | "Vec<MetadataV4>" | "ModuleMetadataV4" | "Option<ModuleMetadataV4>" | "Vec<ModuleMetadataV4>" | "PlainTypeV4" | "Option<PlainTypeV4>" | "Vec<PlainTypeV4>" | "StorageFunctionMetadataV4" | "Option<StorageFunctionMetadataV4>" | "Vec<StorageFunctionMetadataV4>" | "StorageFunctionModifierV4" | "Option<StorageFunctionModifierV4>" | "Vec<StorageFunctionModifierV4>" | "StorageFunctionTypeV4" | "Option<StorageFunctionTypeV4>" | "Vec<StorageFunctionTypeV4>" | "StorageHasherV4" | "Option<StorageHasherV4>" | "Vec<StorageHasherV4>" | "DoubleMapTypeV5" | "Option<DoubleMapTypeV5>" | "Vec<DoubleMapTypeV5>" | "EventMetadataV5" | "Option<EventMetadataV5>" | "Vec<EventMetadataV5>" | "FunctionArgumentMetadataV5" | "Option<FunctionArgumentMetadataV5>" | "Vec<FunctionArgumentMetadataV5>" | "FunctionMetadataV5" | "Option<FunctionMetadataV5>" | "Vec<FunctionMetadataV5>" | "MapTypeV5" | "Option<MapTypeV5>" | "Vec<MapTypeV5>" | "MetadataV5" | "Option<MetadataV5>" | "Vec<MetadataV5>" | "ModuleMetadataV5" | "Option<ModuleMetadataV5>" | "Vec<ModuleMetadataV5>" | "PlainTypeV5" | "Option<PlainTypeV5>" | "Vec<PlainTypeV5>" | "StorageFunctionMetadataV5" | "Option<StorageFunctionMetadataV5>" | "Vec<StorageFunctionMetadataV5>" | "StorageFunctionModifierV5" | "Option<StorageFunctionModifierV5>" | "Vec<StorageFunctionModifierV5>" | "StorageFunctionTypeV5" | "Option<StorageFunctionTypeV5>" | "Vec<StorageFunctionTypeV5>" | "StorageHasherV5" | "Option<StorageHasherV5>" | "Vec<StorageHasherV5>" | "DoubleMapTypeV6" | "Option<DoubleMapTypeV6>" | "Vec<DoubleMapTypeV6>" | "EventMetadataV6" | "Option<EventMetadataV6>" | "Vec<EventMetadataV6>" | "FunctionArgumentMetadataV6" | "Option<FunctionArgumentMetadataV6>" | "Vec<FunctionArgumentMetadataV6>" | "FunctionMetadataV6" | "Option<FunctionMetadataV6>" | "Vec<FunctionMetadataV6>" | "MapTypeV6" | "Option<MapTypeV6>" | "Vec<MapTypeV6>" | "MetadataV6" | "Option<MetadataV6>" | "Vec<MetadataV6>" | "ModuleConstantMetadataV6" | "Option<ModuleConstantMetadataV6>" | "Vec<ModuleConstantMetadataV6>" | "ModuleMetadataV6" | "Option<ModuleMetadataV6>" | "Vec<ModuleMetadataV6>" | "PlainTypeV6" | "Option<PlainTypeV6>" | "Vec<PlainTypeV6>" | "StorageEntryModifierV6" | "Option<StorageEntryModifierV6>" | "Vec<StorageEntryModifierV6>" | "StorageEntryMetadataV6" | "Option<StorageEntryMetadataV6>" | "Vec<StorageEntryMetadataV6>" | "StorageEntryTypeV6" | "Option<StorageEntryTypeV6>" | "Vec<StorageEntryTypeV6>" | "StorageHasherV6" | "Option<StorageHasherV6>" | "Vec<StorageHasherV6>" | "DoubleMapTypeV7" | "Option<DoubleMapTypeV7>" | "Vec<DoubleMapTypeV7>" | "EventMetadataV7" | "Option<EventMetadataV7>" | "Vec<EventMetadataV7>" | "FunctionArgumentMetadataV7" | "Option<FunctionArgumentMetadataV7>" | "Vec<FunctionArgumentMetadataV7>" | "FunctionMetadataV7" | "Option<FunctionMetadataV7>" | "Vec<FunctionMetadataV7>" | "MapTypeV7" | "Option<MapTypeV7>" | "Vec<MapTypeV7>" | "MetadataV7" | "Option<MetadataV7>" | "Vec<MetadataV7>" | "ModuleConstantMetadataV7" | "Option<ModuleConstantMetadataV7>" | "Vec<ModuleConstantMetadataV7>" | "ModuleMetadataV7" | "Option<ModuleMetadataV7>" | "Vec<ModuleMetadataV7>" | "PlainTypeV7" | "Option<PlainTypeV7>" | "Vec<PlainTypeV7>" | "StorageEntryModifierV7" | "Option<StorageEntryModifierV7>" | "Vec<StorageEntryModifierV7>" | "StorageEntryMetadataV7" | "Option<StorageEntryMetadataV7>" | "Vec<StorageEntryMetadataV7>" | "StorageEntryTypeV7" | "Option<StorageEntryTypeV7>" | "Vec<StorageEntryTypeV7>" | "StorageHasherV7" | "Option<StorageHasherV7>" | "Vec<StorageHasherV7>" | "StorageMetadataV7" | "Option<StorageMetadataV7>" | "Vec<StorageMetadataV7>" | "DoubleMapTypeV8" | "Option<DoubleMapTypeV8>" | "Vec<DoubleMapTypeV8>" | "ErrorMetadataV8" | "Option<ErrorMetadataV8>" | "Vec<ErrorMetadataV8>" | "EventMetadataV8" | "Option<EventMetadataV8>" | "Vec<EventMetadataV8>" | "FunctionArgumentMetadataV8" | "Option<FunctionArgumentMetadataV8>" | "Vec<FunctionArgumentMetadataV8>" | "FunctionMetadataV8" | "Option<FunctionMetadataV8>" | "Vec<FunctionMetadataV8>" | "MapTypeV8" | "Option<MapTypeV8>" | "Vec<MapTypeV8>" | "MetadataV8" | "Option<MetadataV8>" | "Vec<MetadataV8>" | "ModuleConstantMetadataV8" | "Option<ModuleConstantMetadataV8>" | "Vec<ModuleConstantMetadataV8>" | "ModuleMetadataV8" | "Option<ModuleMetadataV8>" | "Vec<ModuleMetadataV8>" | "PlainTypeV8" | "Option<PlainTypeV8>" | "Vec<PlainTypeV8>" | "StorageEntryModifierV8" | "Option<StorageEntryModifierV8>" | "Vec<StorageEntryModifierV8>" | "StorageEntryMetadataV8" | "Option<StorageEntryMetadataV8>" | "Vec<StorageEntryMetadataV8>" | "StorageEntryTypeV8" | "Option<StorageEntryTypeV8>" | "Vec<StorageEntryTypeV8>" | "StorageHasherV8" | "Option<StorageHasherV8>" | "Vec<StorageHasherV8>" | "StorageMetadataV8" | "Option<StorageMetadataV8>" | "Vec<StorageMetadataV8>" | "DoubleMapTypeV9" | "Option<DoubleMapTypeV9>" | "Vec<DoubleMapTypeV9>" | "ErrorMetadataV9" | "Option<ErrorMetadataV9>" | "Vec<ErrorMetadataV9>" | "EventMetadataV9" | "Option<EventMetadataV9>" | "Vec<EventMetadataV9>" | "FunctionArgumentMetadataV9" | "Option<FunctionArgumentMetadataV9>" | "Vec<FunctionArgumentMetadataV9>" | "FunctionMetadataV9" | "Option<FunctionMetadataV9>" | "Vec<FunctionMetadataV9>" | "MapTypeV9" | "Option<MapTypeV9>" | "Vec<MapTypeV9>" | "MetadataV9" | "Option<MetadataV9>" | "Vec<MetadataV9>" | "ModuleConstantMetadataV9" | "Option<ModuleConstantMetadataV9>" | "Vec<ModuleConstantMetadataV9>" | "ModuleMetadataV9" | "Option<ModuleMetadataV9>" | "Vec<ModuleMetadataV9>" | "PlainTypeV9" | "Option<PlainTypeV9>" | "Vec<PlainTypeV9>" | "StorageEntryModifierV9" | "Option<StorageEntryModifierV9>" | "Vec<StorageEntryModifierV9>" | "StorageEntryMetadataV9" | "Option<StorageEntryMetadataV9>" | "Vec<StorageEntryMetadataV9>" | "StorageEntryTypeV9" | "Option<StorageEntryTypeV9>" | "Vec<StorageEntryTypeV9>" | "StorageHasherV9" | "Option<StorageHasherV9>" | "Vec<StorageHasherV9>" | "StorageMetadataV9" | "Option<StorageMetadataV9>" | "Vec<StorageMetadataV9>" | "DoubleMapTypeV10" | "Option<DoubleMapTypeV10>" | "Vec<DoubleMapTypeV10>" | "ErrorMetadataV10" | "Option<ErrorMetadataV10>" | "Vec<ErrorMetadataV10>" | "EventMetadataV10" | "Option<EventMetadataV10>" | "Vec<EventMetadataV10>" | "FunctionArgumentMetadataV10" | "Option<FunctionArgumentMetadataV10>" | "Vec<FunctionArgumentMetadataV10>" | "FunctionMetadataV10" | "Option<FunctionMetadataV10>" | "Vec<FunctionMetadataV10>" | "MapTypeV10" | "Option<MapTypeV10>" | "Vec<MapTypeV10>" | "MetadataV10" | "Option<MetadataV10>" | "Vec<MetadataV10>" | "ModuleConstantMetadataV10" | "Option<ModuleConstantMetadataV10>" | "Vec<ModuleConstantMetadataV10>" | "ModuleMetadataV10" | "Option<ModuleMetadataV10>" | "Vec<ModuleMetadataV10>" | "PlainTypeV10" | "Option<PlainTypeV10>" | "Vec<PlainTypeV10>" | "StorageEntryModifierV10" | "Option<StorageEntryModifierV10>" | "Vec<StorageEntryModifierV10>" | "StorageEntryMetadataV10" | "Option<StorageEntryMetadataV10>" | "Vec<StorageEntryMetadataV10>" | "StorageEntryTypeV10" | "Option<StorageEntryTypeV10>" | "Vec<StorageEntryTypeV10>" | "StorageMetadataV10" | "Option<StorageMetadataV10>" | "Vec<StorageMetadataV10>" | "StorageHasherV10" | "Option<StorageHasherV10>" | "Vec<StorageHasherV10>" | "DoubleMapTypeV11" | "Option<DoubleMapTypeV11>" | "Vec<DoubleMapTypeV11>" | "ErrorMetadataV11" | "Option<ErrorMetadataV11>" | "Vec<ErrorMetadataV11>" | "EventMetadataV11" | "Option<EventMetadataV11>" | "Vec<EventMetadataV11>" | "ExtrinsicMetadataV11" | "Option<ExtrinsicMetadataV11>" | "Vec<ExtrinsicMetadataV11>" | "FunctionArgumentMetadataV11" | "Option<FunctionArgumentMetadataV11>" | "Vec<FunctionArgumentMetadataV11>" | "FunctionMetadataV11" | "Option<FunctionMetadataV11>" | "Vec<FunctionMetadataV11>" | "MapTypeV11" | "Option<MapTypeV11>" | "Vec<MapTypeV11>" | "MetadataV11" | "Option<MetadataV11>" | "Vec<MetadataV11>" | "ModuleConstantMetadataV11" | "Option<ModuleConstantMetadataV11>" | "Vec<ModuleConstantMetadataV11>" | "ModuleMetadataV11" | "Option<ModuleMetadataV11>" | "Vec<ModuleMetadataV11>" | "PlainTypeV11" | "Option<PlainTypeV11>" | "Vec<PlainTypeV11>" | "StorageEntryModifierV11" | "Option<StorageEntryModifierV11>" | "Vec<StorageEntryModifierV11>" | "StorageEntryMetadataV11" | "Option<StorageEntryMetadataV11>" | "Vec<StorageEntryMetadataV11>" | "StorageEntryTypeV11" | "Option<StorageEntryTypeV11>" | "Vec<StorageEntryTypeV11>" | "StorageMetadataV11" | "Option<StorageMetadataV11>" | "Vec<StorageMetadataV11>" | "StorageHasherV11" | "Option<StorageHasherV11>" | "Vec<StorageHasherV11>" | "DoubleMapTypeLatest" | "Option<DoubleMapTypeLatest>" | "Vec<DoubleMapTypeLatest>" | "EventMetadataLatest" | "Option<EventMetadataLatest>" | "Vec<EventMetadataLatest>" | "ExtrinsicMetadataLatest" | "Option<ExtrinsicMetadataLatest>" | "Vec<ExtrinsicMetadataLatest>" | "FunctionArgumentMetadataLatest" | "Option<FunctionArgumentMetadataLatest>" | "Vec<FunctionArgumentMetadataLatest>" | "FunctionMetadataLatest" | "Option<FunctionMetadataLatest>" | "Vec<FunctionMetadataLatest>" | "MapTypeLatest" | "Option<MapTypeLatest>" | "Vec<MapTypeLatest>" | "MetadataLatest" | "Option<MetadataLatest>" | "Vec<MetadataLatest>" | "ModuleConstantMetadataLatest" | "Option<ModuleConstantMetadataLatest>" | "Vec<ModuleConstantMetadataLatest>" | "ModuleMetadataLatest" | "Option<ModuleMetadataLatest>" | "Vec<ModuleMetadataLatest>" | "PlainTypeLatest" | "Option<PlainTypeLatest>" | "Vec<PlainTypeLatest>" | "StorageEntryMetadataLatest" | "Option<StorageEntryMetadataLatest>" | "Vec<StorageEntryMetadataLatest>" | "StorageEntryModifierLatest" | "Option<StorageEntryModifierLatest>" | "Vec<StorageEntryModifierLatest>" | "StorageEntryTypeLatest" | "Option<StorageEntryTypeLatest>" | "Vec<StorageEntryTypeLatest>" | "StorageMetadataLatest" | "Option<StorageMetadataLatest>" | "Vec<StorageMetadataLatest>" | "StorageHasher" | "Option<StorageHasher>" | "Vec<StorageHasher>" | "MetadataAll" | "Option<MetadataAll>" | "Vec<MetadataAll>" | "ApiId" | "Option<ApiId>" | "Vec<ApiId>" | "BlockHash" | "Option<BlockHash>" | "Vec<BlockHash>" | "ChainProperties" | "Option<ChainProperties>" | "Vec<ChainProperties>" | "ExtrinsicOrHash" | "Option<ExtrinsicOrHash>" | "Vec<ExtrinsicOrHash>" | "ExtrinsicStatus" | "Option<ExtrinsicStatus>" | "Vec<ExtrinsicStatus>" | "Health" | "Option<Health>" | "Vec<Health>" | "KeyValueOption" | "Option<KeyValueOption>" | "Vec<KeyValueOption>" | "NetworkState" | "Option<NetworkState>" | "Vec<NetworkState>" | "PeerInfo" | "Option<PeerInfo>" | "Vec<PeerInfo>" | "RpcMethods" | "Option<RpcMethods>" | "Vec<RpcMethods>" | "RuntimeDispatchInfo" | "Option<RuntimeDispatchInfo>" | "Vec<RuntimeDispatchInfo>" | "RuntimeVersionApi" | "Option<RuntimeVersionApi>" | "Vec<RuntimeVersionApi>" | "RuntimeVersion" | "Option<RuntimeVersion>" | "Vec<RuntimeVersion>" | "StorageChangeSet" | "Option<StorageChangeSet>" | "Vec<StorageChangeSet>" | "IdentityId" | "Option<IdentityId>" | "Vec<IdentityId>" | "Ticker" | "Option<Ticker>" | "Vec<Ticker>" | "Document" | "Option<Document>" | "Vec<Document>" | "AssetType" | "Option<AssetType>" | "Vec<AssetType>" | "IdentifierType" | "Option<IdentifierType>" | "Vec<IdentifierType>" | "SecurityToken" | "Option<SecurityToken>" | "Vec<SecurityToken>" | "LinkedKeyInfo" | "Option<LinkedKeyInfo>" | "Vec<LinkedKeyInfo>" | "AccountKey" | "Option<AccountKey>" | "Vec<AccountKey>" | "Permission" | "Option<Permission>" | "Vec<Permission>" | "Link" | "Option<Link>" | "Vec<Link>" | "LinkData" | "Option<LinkData>" | "Vec<LinkData>" | "SignatoryType" | "Option<SignatoryType>" | "Vec<SignatoryType>" | "Signatory" | "Option<Signatory>" | "Vec<Signatory>" | "SigningItem" | "Option<SigningItem>" | "Vec<SigningItem>" | "SigningItemWithAuth" | "Option<SigningItemWithAuth>" | "Vec<SigningItemWithAuth>" | "IdentityRole" | "Option<IdentityRole>" | "Vec<IdentityRole>" | "PreAuthorizedKeyInfo" | "Option<PreAuthorizedKeyInfo>" | "Vec<PreAuthorizedKeyInfo>" | "DidRecord" | "Option<DidRecord>" | "Vec<DidRecord>" | "Claim" | "Option<Claim>" | "Vec<Claim>" | "ClaimMetaData" | "Option<ClaimMetaData>" | "Vec<ClaimMetaData>" | "ClaimValue" | "Option<ClaimValue>" | "Vec<ClaimValue>" | "ClaimRecord" | "Option<ClaimRecord>" | "Vec<ClaimRecord>" | "DataTypes" | "Option<DataTypes>" | "Vec<DataTypes>" | "AssetRule" | "Option<AssetRule>" | "Vec<AssetRule>" | "Operators" | "Option<Operators>" | "Vec<Operators>" | "RuleData" | "Option<RuleData>" | "Vec<RuleData>" | "STO" | "Option<STO>" | "Vec<STO>" | "Investment" | "Option<Investment>" | "Vec<Investment>" | "SimpleTokenRecord" | "Option<SimpleTokenRecord>" | "Vec<SimpleTokenRecord>" | "FeeOf" | "Option<FeeOf>" | "Vec<FeeOf>" | "Dividend" | "Option<Dividend>" | "Vec<Dividend>" | "TargetIdAuthorization" | "Option<TargetIdAuthorization>" | "Vec<TargetIdAuthorization>" | "TickerRegistration" | "Option<TickerRegistration>" | "Vec<TickerRegistration>" | "TickerRegistrationConfig" | "Option<TickerRegistrationConfig>" | "Vec<TickerRegistrationConfig>" | "SignData" | "Option<SignData>" | "Vec<SignData>" | "Motion" | "Option<Motion>" | "Vec<Motion>" | "Ballot" | "Option<Ballot>" | "Vec<Ballot>" | "MipsMetadata" | "Option<MipsMetadata>" | "Vec<MipsMetadata>" | "PolymeshVotes" | "Option<PolymeshVotes>" | "Vec<PolymeshVotes>" | "MipsIndex" | "Compact<MipsIndex>" | "Option<MipsIndex>" | "Vec<MipsIndex>" | "MipsPriority" | "Option<MipsPriority>" | "Vec<MipsPriority>" | "MIP" | "Option<MIP>" | "Vec<MIP>" | "PolymeshReferendumInfo" | "Option<PolymeshReferendumInfo>" | "Vec<PolymeshReferendumInfo>" | "TickerTransferApproval" | "Option<TickerTransferApproval>" | "Vec<TickerTransferApproval>" | "OffChainSignature" | "Option<OffChainSignature>" | "Vec<OffChainSignature>" | "PermissionedValidator" | "Option<PermissionedValidator>" | "Vec<PermissionedValidator>" | "Authorization" | "Option<Authorization>" | "Vec<Authorization>" | "AuthorizationData" | "Option<AuthorizationData>" | "Vec<AuthorizationData>" | "Compliance" | "Option<Compliance>" | "Vec<Compliance>" | "SmartExtensionType" | "Option<SmartExtensionType>" | "Vec<SmartExtensionType>" | "SmartExtension" | "Option<SmartExtension>" | "Vec<SmartExtension>" | "ProportionMatch" | "Option<ProportionMatch>" | "Vec<ProportionMatch>" | "AuthorizationNonce" | "Compact<AuthorizationNonce>" | "Option<AuthorizationNonce>" | "Vec<AuthorizationNonce>" | "Counter" | "Compact<Counter>" | "Option<Counter>" | "Vec<Counter>" | "Commission" | "Option<Commission>" | "Vec<Commission>">(type: K, ...params: any[]) => InterfaceRegistry[K];
    /**
     * @description Disconnect from the underlying provider, halting all network traffic
     */
    disconnect(): void;
    /**
     * @description Finds the definition for a specific [[CallFunction]] based on the index supplied
     */
    findCall(callIndex: Uint8Array | string): CallFunction;
    /**
     * @description Finds the definition for a specific [[RegistryError]] based on the index supplied
     */
    findError(errorIndex: Uint8Array | string): RegistryError;
    /**
     * @description Register additional user-defined of chain-specific types in the type registry
     */
    registerTypes(types?: RegistryTypes): void;
    /**
     * @description Set an external signer which will be used to sign extrinsic when account passed in is not KeyringPair
     */
    setSigner(signer: Signer): void;
    /**
     * @description Signs a raw signer payload, string or Uint8Array
     */
    sign(address: KeyringSigner | string, data: SignerPayloadRawBase, { signer }?: SignerRawOptions): Promise<string>;
}
export {};
